#include <stdio.h>
#include "il_image.h"
#include "il_meta.h"
#include "il_opcodes.h"
#include "il_program.h"
#include "il_dumpasm.h"

ILImage * LoadImage(char *filename, ILContext *context, int flags)
{
	int loadError;
  	ILImage *image;
  	FILE *stream;
  	/*attempt to open file */
  	stream = fopen (filename, "rb");
  	if (stream == NULL)
    	stream = fopen (filename, "r");	//just in case you use an old libc
  	if (stream == NULL)
    {
    	fprintf (stderr, "file missing:%s\n", filename);
    	return NULL;		//just make sure the pointer is set to NULL
   	}
  	/* Attempt to load the image into memory */
  	loadError = ILImageLoad (stream, filename, context, &image, flags);
  	if (loadError != 0)
    {
      	fprintf (stderr, "%s: %s\n", filename, ILImageLoadError (loadError));
      	return NULL;
    }

  /* Loaded and ready to go */
  	return image;
}

static void ProcessMethod(ILImage *img, ILMethod *method)
{
	ILMethodCode code;
	ILClass* owner=ILMethod_Owner(method);
	int i;
	if(!ILMethodGetCode(method,&code))
	{
		return ; //might be abstract method or illegal code
	}
	printf("\n*****************************************************************\n");
	printf("%s",ILClass_Namespace(owner) ? ILClass_Namespace(owner) : "");
	printf("%c",ILClass_Namespace(owner) ? '.' : 0);
	printf("%s.%s\n",ILClass_Name(owner),ILMethod_Name(method));
	printf("\n\t");
	for(i=0;i<code.codeLen;i++)
	{
		printf("%02x ",((unsigned char)((unsigned char*)code.code)[i]));
		if((i  & 15) == 15)printf("\n\t");
	}
	printf("\n*****************************************************************\n");
}

static void ProcessAllMethods(ILImage *img)
{
  	unsigned long methods;
  	ILToken token;		//int 32
  	ILMethod *data;
  	methods = ILImageNumTokens (img, IL_META_TOKEN_METHOD_DEF);
  	for (token = 1; token <= methods; token++)
    {
      	data=(ILMethod*)(ILImageTokenInfo(img,IL_META_TOKEN_METHOD_DEF|token));
		if(data)
		{
			ProcessMethod(img,data);
		}
    }
}

int main(int argc,char *argv[])
{
	char * fname;
	ILContext *context;
	ILImage *img;
	if(argc<=1)
	{
		fprintf(stderr,"need more args !\n");
		return 1;
	}
	fname=argv[1];
	context=ILContextCreate(); // context for loading .dlls 
	img=LoadImage(fname,context,
							IL_LOADFLAG_FORCE_32BIT |
							IL_LOADFLAG_PRE_VALIDATE |
							IL_LOADFLAG_NO_RESOLVE
						);
	if(img==NULL)
	{
		return 1;
	}
	ProcessAllMethods(img);
	return 0;
}
